/*

Copyright (C) 2009 CUVELIER Thibaut

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

*/

//Mines
#include "devil-qt-handler.hpp"
//Qt
#include <QtGui/QImage>
#include <QtCore/QIODevice>
#include <QtCore/QVariant>
#include <QtCore/QDataStream>
//DevIL
#include <IL/il.h>
#include <IL/ilu.h>	//seule la correction gamma nous impose cet header

/*
**	Initialise DevIL et autorise l'crasement des fichiers 
**
**	Cre le Lump qui contiendra l'image en mmoire
*/
DevILHandler::DevILHandler()
{
	ilInit();
	iluInit();
}

/*
**	Pas de "dsinitialisation"
*/
DevILHandler::~DevILHandler()
{
}

/*
**	Prcise le format
*/
void DevILHandler::setFormat(const QByteArray & format)
{
	this->format = format;
		 if(format == "cut")
		this->I_Format = IL_CUT;
	else if(format == "dcx")
		this->I_Format = IL_DCX;
	else if(format == "exr")
		this->I_Format = IL_EXR;
	else if(format == "hdr")
		this->I_Format = IL_HDR;
	else if(format == "ico")
		this->I_Format = IL_ICO;
	else if(format == "cur")
		this->I_Format = IL_ICO;
	else if(format == "icns")
		this->I_Format = IL_ICNS;
	else if(format == "gif")
		this->I_Format = IL_GIF;
	else if(format == "jp2")
		this->I_Format = IL_JP2;
	else if(format == "lif")
		this->I_Format = IL_LIF;
	else if(format == "pcd")
		this->I_Format = IL_PCD;
	else if(format == "pic")
		this->I_Format = IL_PIC;
	else if(format == "psd")
		this->I_Format = IL_PSD;
	else if(format == "pdd")
		this->I_Format = IL_PSD;
	else if(format == "psp")
		this->I_Format = IL_PSP;
	else if(format == "wal")
		this->I_Format = IL_WAL;
	else if(format == "vtf")
		this->I_Format = IL_VTF;
	else if(format == "bmp")
		this->I_Format = IL_BMP;
	else if(format == "dib")
		this->I_Format = IL_BMP;
	else if(format == "dds")
		this->I_Format = IL_DDS;
	else if(format == "jpg")
		this->I_Format = IL_JPG;
	else if(format == "jpe")
		this->I_Format = IL_JPG;
	else if(format == "jpeg")
		this->I_Format = IL_JPG;
	else if(format == "pal")
		this->I_Format = IL_JASC_PAL;
	else if(format == "pcx")
		this->I_Format = IL_PCX;
	else if(format == "png")
		this->I_Format = IL_PNG;
	else if(format == "pnm")
		this->I_Format = IL_PNM;
	else if(format == "pgm")
		this->I_Format = IL_PNM;
	else if(format == "pnm")
		this->I_Format = IL_PNM;
	else if(format == "ppm")
		this->I_Format = IL_PNM;
	else if(format == "raw")
		this->I_Format = IL_RAW;
	else if(format == "sgi")
		this->I_Format = IL_SGI;
	else if(format == "bw")
		this->I_Format = IL_SGI;
	else if(format == "rgb")
		this->I_Format = IL_SGI;
	else if(format == "rgba")
		this->I_Format = IL_SGI;
	else if(format == "tif")
		this->I_Format = IL_TIF;
	else if(format == "tiff")
		this->I_Format = IL_TIF;
	else if(format == "tga")
		this->I_Format = IL_TGA;
	else if(format == "vda")
		this->I_Format = IL_TGA;
	else if(format == "icb")
		this->I_Format = IL_TGA;
	else if(format == "vst")
		this->I_Format = IL_TGA;
	else
		this->I_Format = IL_TYPE_UNKNOWN;
}

/*
**	Rcupre le format
*/
ILenum DevILHandler::format()
		const
{
	return this->I_Format;
}

QByteArray DevILHandler::format()
		const
{
	return this->format;
}

/*
**	Rcupre le QIODevice  partir duquel tout est effectu
*/
QIODevice * DevILHandler::device()
		const
{
	return this->Device;
}

/*
**	Prcise le priphrique  partir duquel la lecture sera effectue
**	Pas de retour autoris => pas de retour, mme en cas d'erreur ...
*/
void DevILHandler::setDevice(QIODevice *device)
{
	this->Size		= Device->size		();
	this->Lump		= (char *) malloc	(Size);
					  ilLoadL			(IL_TYPE_UNKNOWN, this->Lump, this->Size);
	this->Width		= ilGetInteger		(IL_IMAGE_WIDTH);
	this->Height	= ilGetInteger		(IL_IMAGE_HEIGHT);
	this->Depth		= ilGetInteger		(IL_IMAGE_DEPTH);
	this->Size_wh	= QSize				(this->Width, this->Height);
	this->Palette	= ilGetInteger		(IL_PALETTE_BASE_TYPE);
	this->NumImages = ilGetInteger		(IL_NUM_IMAGES);
	this->DurImage	= ilGetInteger		(IL_IMAGE_DURATION);
	this->Gamma		= 1.0;
	this->Data		= ilGetData			();
	//Dtermination du QImage::Format depuis la palette de DevIL (tout en dur)
	     if	(this->Palette == IL_RGB)
					this->Q_Format = QImage::Format_RGB32;
	else if	(this->Palette == IL_RGBA)
					this->Q_Format = QImage::Format_ARGB32;
	else if (this->Palette = IL_COLOUR_INDEX)
					this->Q_Format = QImage::Format_Indexed8;
	else if (this->Palette = IL_COLOR_INDEX)
					this->Q_Format = QImage::Format_Indexed8;
	else if (this->Palette == IL_BGR)
					this->Q_Format = QImage::Format_Invalid;	// peut-tre, un jour, support par Qt
	else if (this->Palette == IL_BGRA)
					this->Q_Format = QImage::Format_Invalid;	// peut-tre, un jour, support par Qt
	else if (this->Palette == IL_ALPHA)
					this->Q_Format = QImage::Format_Invalid;	// peut-tre, un jour, support par Qt
	else if (this->Palette == IL_LUMINANCE)
					this->Q_Format = QImage::Format_Invalid;	// peut-tre, un jour, support par Qt
	else if (this->Palette == IL_LUMINANCE_ALPHA)
					this->Q_Format = QImage::Format_Invalid;	// peut-tre, un jour, support par Qt
	else
					this->Q_Format = QImage::Format_Invalid;	//si DevIL lui-mme n'en sait rien ...
}
void DevILHandler::setDevice(QFile *device)
{
	this->fDevice	= device;
	this->fileName	= this->fDevice->fileName().toStdString().c_str();
	ilLoad			  (IL_TYPE_UNKNOWN, this->fileName);
	this->Width		= ilGetInteger		(IL_IMAGE_WIDTH);
	this->Height	= ilGetInteger		(IL_IMAGE_HEIGHT);
	this->Depth		= ilGetInteger		(IL_IMAGE_DEPTH);
	this->Size_wh	= QSize				(this->Width, this->Height);
	this->Palette	= ilGetInteger		(IL_PALETTE_BASE_TYPE);
	this->NumImages = ilGetInteger		(IL_NUM_IMAGES);
	this->DurImage	= ilGetInteger		(IL_IMAGE_DURATION);
	this->Gamma		= 1.0;
	this->Data		= ilGetData			();
	
	//Dtermination du QImage::Format depuis la palette de DevIL (tout en dur)
	     if	(this->Palette == IL_RGB)
			this->Q_Format = QImage::Format_RGB32;
	else if	(this->Palette == IL_RGBA)
			this->Q_Format = QImage::Format_ARGB32;
	else if (this->Palette = IL_COLOUR_INDEX)
			this->Q_Format = QImage::Format_Indexed8;
	else if (this->Palette = IL_COLOR_INDEX)
			this->Q_Format = QImage::Format_Indexed8;
	/*
	else if (this->Palette == IL_BGR)
			this->Q_Format = QImage::Format_Invalid;	// peut-tre, un jour, support par Qt
	else if (this->Palette == IL_BGRA)
			this->Q_Format = QImage::Format_Invalid;	// peut-tre, un jour, support par Qt
	else if (this->Palette == IL_ALPHA)
			this->Q_Format = QImage::Format_Invalid;	// peut-tre, un jour, support par Qt
	else if (this->Palette == IL_LUMINANCE)
			this->Q_Format = QImage::Format_Invalid;	// peut-tre, un jour, support par Qt
	else if (this->Palette == IL_LUMINANCE_ALPHA)
			this->Q_Format = QImage::Format_Invalid;	// peut-tre, un jour, support par Qt
	*/
	else
			this->Q_Format = QImage::Format_Invalid;	// si DevIL lui-mme n'en sait rien ...
}

/*
**	Utilisent les fonctions prvues de DevIL pour la validit des fichiers (extensions + format)
**	et de Qt (lecture possible ?)
*/
bool DevILHandler::canRead()
		const
/*
--	On ne sait rien du priphrique : qu'est-ce que l'on peut en dire ?!
--	S'il est dj charg, alors on peut en dire quelque chose
*/
{
	if(this->device() == 0);
	{
		qWarning("DevILHandler::canRead() called with no device");
		return false;
	}

	return ilIsValidL(IL_TYPE_UNKNOWN, this->Lump, this->Size);
}
/*
--	On a le priphrique, mais aucun moyen d'accs direct ... C'est parti pour la triche !
*/
bool DevILHandler::canRead(QIODevice *device)
{
	if (!device)
	{
		qWarning("DevILHandler::canRead() called with no device or an unusable device");
		return false;
	}

	this->setDevice(device);

	/*
	--	DevIL connat enfin le contenu et la taille du fichier charg du QIODevice (ouf !)
	--	Surtout, le lump est du bon format ...
	--
	--	Maintenant, on vrifie le format (que l'on ne connat toujours pas ...)
	*/

	return ilIsValidL(IL_TYPE_UNKNOWN, this->Lump, this->Size);
}
/*
--	Si on a un QFile, on sait tout ce que l'on veut du fichier (youpi !)
*/
bool DevILHandler::canRead(QFile *device)
{
	if (!device)
	{
		qWarning("DevILHandler::canRead() called with no device or an unusable file");
		return false;
	}

	this->setDevice(device);

	/*
	--	DevIL connat enfin le contenu et la taille du fichier charg du QIODevice (ouf !)
	--	Surtout, le Lump est du bon format ...
	--
	--	Maintenant, on vrifie le format (que l'on ne connat toujours pas ...)
	*/

	return ilIsValid(IL_TYPE_UNKNOWN, this->fileName);
}

/*
**	Lit l'image
**	Image forcment dj charge prcdemment, par canRead (donc setDevice)
*/
bool DevILHandler::read (QImage *image)
{
	this->Image = QImage (this->Size_wh, this->Q_Format);
	this->Image.fromData(this->Data, this->Size, 0);
	return true;
}

/*
**	crit l'image
*/
bool DevILHandler::write (const QImage & image)
{
	if(! this->Device->isWritable())
		return false;
	//nous ne connaissons pas d'avance la taille du fichier rsultant ==> 0
	if(ilSaveL(this->I_Format, this->Lump, 0) == 0)
		return false;
	if(this->Device->write(this->Lump) == -1)
		return false;
	return true;
}

/*
**	Gestion des images animes (GIF...)
*/
int DevILHandler::imageCount()
		const
{
	if(this->I_Format != IL_GIF)
		return 0;
	else
		return this->NumImages;
}

int DevILHandler::loopCount()
		const
{
	if(this->I_Format != IL_GIF)
		return 0;
	else
		return 1;
}

int DevILHandler::nextImageDelay()
		const
{
	if(this->I_Format != IL_GIF)
		return 0;
	else
		return this->DurImage;
}

bool DevILHandler::jumpToImage(int imageNumber)
		const
{
	return false;
}

/*
**	Gestion des ImageOption
*/
QVariant DevILHandler::option(QImageIOHandler::ImageOption option)
		const
{
		 if(option == QImageIOHandler::Size)
		return this->Size_wh;	//unsettable
/*
	else if (option == QImageIOHandler::ClipRect)
		return false;
	else if (option == QImageIOHandler::ScaledSize)
		return false;
	else if (option == QImageIOHandler::ScaledClipRect)
		return false;
	else if (option == QImageIOHandler::Description)
		return false;
	else if (option == QImageIOHandler::CompressionRatio)
		return false;
*/
	else if (option == QImageIOHandler::Gamma)
		return this->Gamma;	//settable
/*
	else if (option == QImageIOHandler::Quality)
		return false;
	else if (option == QImageIOHandler::Name)
		return false;
	else if (option == QImageIOHandler::SubType)
		return false;
	else if (option == QImageIOHandler::IncrementalReading)
		return false;
	else if (option == QImageIOHandler::Endianness)
		return false;
*/
	else if (option == QImageIOHandler::Animation)
		if(this->I_Format == IL_GIF)
			return true;
		else
			return false;
/*
	else if (option == QImageIOHandler::BackgroundColor)
		return false;
*/
	else
		return false;
}

void DevILHandler::setOption(QImageIOHandler::ImageOption option, const QVariant & value)
{
	if (option == QImageIOHandler::Gamma)
	{
		this->Gamma = value.toDouble();
		iluGammaCorrect(this->Gamma);
	}
}

bool DevILHandler::supportsOption(QImageIOHandler::ImageOption option)
		const
{
		 if(option == QImageIOHandler::Size)
		return true;	//unsettable
/*
	else if (option == QImageIOHandler::ClipRect)
		return false;
	else if (option == QImageIOHandler::ScaledSize)
		return false;
	else if (option == QImageIOHandler::ScaledClipRect)
		return false;
	else if (option == QImageIOHandler::Description)
		return false;
	else if (option == QImageIOHandler::CompressionRatio)
		return false;
*/
	else if (option == QImageIOHandler::Gamma)	// iluGammaCorrect (1.0) == image ; iluGammaCorrect (0.0) = black ; iluGammaCorrect (10.0) = white
		return true;	//settable
/*
	else if (option == QImageIOHandler::Quality)
		return false;
	else if (option == QImageIOHandler::Name)
		return false;
	else if (option == QImageIOHandler::SubType)
		return false;
	else if (option == QImageIOHandler::IncrementalReading)
		return false;
	else if (option == QImageIOHandler::Endianness)
		return false;
*/
	else if (option == QImageIOHandler::Animation)
		return true;	//unsettable
/*
	else if (option == QImageIOHandler::BackgroundColor)
		return false;
*/
	else
		return false;
}
