/*

Copyright (C) 2009 CUVELIER Thibaut

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

*/

#ifndef DEVIL_QT_HANDLER_HPP
#define DEVIL_QT_HANDLER_HPP

//Qt
#include <QtGui/QImage>
#include <QtGui/QImageIOHandler>
#include <QtGui/QTransform>
#include <QtGui/QPaintDevice>
#include <QtGui/QRgb>
#include <QtCore/QByteArray>
#include <QtCore/QRect>
#include <QtCore/QString>
#include <QtCore/QFile>
//DevIL
#include <IL/il.h>

class DevILHandler : public QImageIOHandler
{
public:
						DevILHandler	();
						~DevILHandler	();

			void		setDevice		(QIODevice *device);
			void		setDevice		(QFile *device);
			QIODevice *	device			()
								const;

			bool		canRead			()
								const;
			bool		canRead			(QIODevice *device);
			bool		canRead			(QFile *device);

			bool		read			(QImage *image);
			bool		write			(const QImage &image);

			QVariant	option			(ImageOption option)
								const;
			void		setOption		(ImageOption option, const QVariant &value);
			bool		supportsOption	(ImageOption option)
								const;

			int			loopCount() 
								const;
			int			nextImageDelay() 
								const;
			int			imageCount() 
								const;
			bool		jumpToImage(int imageNumber)
								const;

			void		setFormat		(const QByteArray & format);
			ILenum		format			()
								const;
			QByteArray	format			()
								const;

private:
			char *			Lump;
			ILuint			Size;	//de l'image en octets
			ILuint			Width;
			ILuint			Height;
			ILuint			Depth;
			ILubyte	*		Data;
			ILenum			I_Format;
			QByteArray		format;
			ILuint			Palette;
			ILuint			NumImages;
			ILuint			DurImage;
			float			Gamma;
	const	char *			fileName;
			QIODevice *		Device;
			QFile *			fDevice;
			QSize			Size_wh;	//de l'image en pixels (hauteur x largeur)
			QImage			Image;
			QImage::Format	Q_Format;
};
#endif //DEVIL_QT_HANDLER_HPP