/*

Copyright (C) 2009 CUVELIER Thibaut

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

*/

#include "devil-qt-plugin.hpp"

DevILPlugin::DevILPlugin()  {}
DevILPlugin::~DevILPlugin() {}

/*
**	Retourne la liste des extensions des formats supports
**	Voir http://openil.sourceforge.net/features.php
*/
QStringList DevILPlugin::keys()
		const
{
	QStringList	supported;
	supported << "act";
	supported << "bmp";
	supported << "cut";
	supported << "dcx";
	supported << "dds";
	supported << "exr";
	supported << "hdr";
	supported << "ico";
	supported << "icns";
	supported << "gif";
	supported << "jpg";
	supported << "jp2";
	supported << "lbm";
	supported << "lif";
	supported << "mdp";
	supported << "pal";
	supported << "pcd";
	supported << "pcx";
	supported << "pic";
	supported << "png";
	supported << "pnm";
	supported << "psd";
	supported << "psp";
	supported << "raw";
	supported << "sgi";
	supported << "tga";
	supported << "tif";
	supported << "wal";
	supported << "vtf";
	return supported;
}

/*
**	Retourne les capacits (lecture ou criture) en fonction du format (extension)
**	Voir http://openil.sourceforge.net/features.php
*/
QImageIOPlugin::Capabilities DevILPlugin::capabilities
	(QIODevice *device, const QByteArray &format)
		const
{
	QImageIOPlugin::Capabilities cap;

	/*
	-- On ne connat pas l'extension : comment peut-on savoir si c'est support ?
	-- On retourne donc les capacits, vides.
	*/
	if (format.isEmpty()
	|| !( device->isOpen() )
	   )
		
		return cap;

	/*
	-- Si l'extension est supporte en criture, elle l'est en lecture (dans le cas de DevIL)
	*/
	if(		format == "bmp"		//BMP
		||	format == "dib"		//BMP
		||	format == "dds"
		||	format == "jpg"		//JPG
		||	format == "jpe"		//JPG
		||	format == "jpeg"	//JPEG
		||	format == "pal"
		||	format == "pcx"
		||	format == "png"
		||	format == "pnm"		//PNM
		||	format == "pgm"		//PNM
		||	format == "pnm"		//PNM
		||	format == "ppm"		//PNM
		||	format == "raw"
		||	format == "sgi"		//SGI
		||	format == "bw"		//SGI
		||	format == "rgb"		//SGI
		||	format == "rgba"	//SGI
		||	format == "tif"		//TIFF
		||	format == "tiff"	//TIFF
		||	format == "tga"		//TGA
		||	format == "vda"		//TGA
		||	format == "icb"		//TGA
		||	format == "vst"		//TGA
		)
	{
		cap |= CanRead;
		cap |= CanWrite;
		return cap;
	}
	/*
	-- L'extension est-elle supporte uniquement en lecture ?
	*/
	else if (format == "cut"
		||	 format == "dcx"
		||	 format == "exr"
		||	 format == "hdr"
		||	 format == "ico"	//ICO
		||	 format == "cur"	//ICO
		||	 format == "icns"
		||	 format == "gif"
		||	 format == "jpg"
		||	 format == "jp2"
		||	 format == "lbm"
		||	 format == "lif"
		||	 format == "pcd"
		||	 format == "pic"
		||	 format == "psd"	//PSD
		||	 format == "pdd"	//PSD
		||	 format == "psp"
		||	 format == "wal"
		||	 format == "vtf"	)
	{
		cap |= CanRead;
		return cap;
	}
	/*
	-- Supporte nulle-part ? Non supporte !
	*/
	else
	{
		return cap;
	}
}

/*
**	Retourne une instance du handler, en prcisant le device (fichier  lire) et le format (extension)
*/
QImageIOHandler * DevILPlugin::create(QIODevice *device, const QByteArray &format)
		const
{
    QImageIOHandler *handler = new DevILHandler;
    handler->setDevice(device);
    handler->setFormat(format);
    return handler;
}

Q_EXPORT_STATIC_PLUGIN	(				DevILPlugin	)
Q_EXPORT_PLUGIN2		(devil-qt,		DevILPlugin	)
